import time
from typing import List, Optional

from nonebot.adapters.onebot.v11 import Bot
from pydantic import BaseModel

from nekro_agent.core.bot import get_bot
from nekro_agent.core.config import config
from nekro_agent.core.logger import logger

MAX_PRESET_STATUS_LIST_SIZE = 99
MAX_PRESET_STATUS_SHOW_SIZE = 24


class PresetStatus(BaseModel):
    """预设状态"""

    setting_name: str
    description: str
    translated_timestamp: int


class channelData(BaseModel):
    chat_key: str
    preset_status_list: List[PresetStatus] = []

    def append_preset_status(self, preset_status: PresetStatus):
        self.preset_status_list.append(preset_status)
        if len(self.preset_status_list) > MAX_PRESET_STATUS_LIST_SIZE:
            self.preset_status_list.pop(0)

    async def update_preset_status(self, preset_status: PresetStatus):
        latest_preset_status: Optional[PresetStatus] = self.get_latest_preset_status()
        if latest_preset_status is not None and latest_preset_status.setting_name == preset_status.setting_name:
            self.preset_status_list[-1].translated_timestamp = preset_status.translated_timestamp
        else:
            self.append_preset_status(preset_status)
            chat_type, chat_id = self.chat_key.split("_")
            try:
                await get_bot().set_group_card(
                    group_id=int(chat_id),
                    user_id=int(config.BOT_QQ),
                    card=f"{config.AI_NAME_PREFIX}{preset_status.setting_name}",
                )
            except Exception as e:
                logger.warning(f"会话 {self.chat_key} 尝试更新群名片失败: {e}")

    def get_latest_preset_status(self) -> Optional[PresetStatus]:
        if len(self.preset_status_list) == 0:
            return None
        return self.preset_status_list[-1]

    def render_prompt(self) -> str:
        history_str = ""
        if len(self.preset_status_list) > 1:
            history_str = "History Status:\n "
            for preset_status in self.preset_status_list[-MAX_PRESET_STATUS_SHOW_SIZE:-1]:
                time_diff_str = time.strftime("%H:%M:%S", time.gmtime(time.time() - preset_status.translated_timestamp))
                history_str += f"- {preset_status.setting_name}: {preset_status.description} ({time_diff_str} ago)\n"
            history_str = history_str.strip() + "\n"

        latest_preset_status = self.get_latest_preset_status()
        if latest_preset_status is None:
            return "Current Character Setting status: No special status. (Use `update_preset_status` to update it **immediately**!)"
        time_diff = time.time() - latest_preset_status.translated_timestamp
        time_diff_str = time.strftime("%H:%M:%S", time.gmtime(time_diff))
        addition_str = (
            "Please Use `update_preset_status` to update it."
            if time_diff > 300
            else "Use `update_preset_status` to update it If doesn't fit the current scene description."
        )
        return f"{history_str}Current Character Setting status: {latest_preset_status.setting_name}: {latest_preset_status.description} (updated {time_diff_str} ago. {addition_str})"

    def clear_preset_status_list(self):
        self.preset_status_list = []



import time
from typing import Optional

from nekro_agent.core import logger
from nekro_agent.models.db_chat_channel import DBChatChannel
from nekro_agent.schemas.agent_ctx import AgentCtx
from nekro_agent.schemas.agent_message import (
    AgentMessageSegment,
    AgentMessageSegmentType,
)
from nekro_agent.schemas.chat_channel import PresetStatus, channelData
from nekro_agent.services.chat import chat_service
from nekro_agent.services.extension import ExtMetaData
from nekro_agent.tools.collector import MethodType, agent_collector
from nekro_agent.tools.common_util import (
    convert_file_name_to_container_path,
    download_file,
)

__meta__ = ExtMetaData(
    name="basic",
    description="Nekro-Agent 交互基础工具集",
    version="0.1.0",
    author="KroMiose",
    url="https://github.com/KroMiose/nekro-agent",
)

# ========================================================================================
# |                              Nekro-Agent 交互基础工具集                                |
# ========================================================================================
#   扩展编写注意:
#     1. 所有注解会被 AI 引用时参考，请务必准确填写
#     2. _ctx: AgentCtx 中存储有关当前会话的上下文信息，不需要也不能加入到注释，以免误导 AI
#     3. _ctx 参数务必放在最后，以免因 AI 使用传递不规范导致错误
#     4. 如果需要在注解中编写应用示例等信息，务必不要体现 _ctx 的存在，并且使用 `同步调用` 的方式
#        (即不需要 `await func()` )，因为其实际执行是通过 rpc 进行的
# ========================================================================================


@agent_collector.mount_method(MethodType.TOOL)
async def send_msg_text(chat_key: str, message: str, _ctx: AgentCtx) -> bool:
    """发送聊天消息文本

    Args:
        chat_key (str): 会话标识
        message (str): 消息内容

    Returns:
        bool: 是否发送成功
    """
    try:
        message_ = [AgentMessageSegment(content=message)]
        await chat_service.send_agent_message(chat_key, message_, _ctx, record=True)
    except Exception as e:
        logger.exception(f"Error sending message to chat: {e}")
        return False
    else:
        return True


@agent_collector.mount_method(MethodType.TOOL)
async def send_msg_img(chat_key: str, file_path: str, _ctx: AgentCtx) -> bool:
    """发送聊天消息图片资源 (支持资源格式: jpg, jpeg, png, gif, bmp; 不支持的格式请先转换)

    Args:
        chat_key (str): 会话标识
        file_path (str): 图片路径或 URL

    Returns:
        bool: 是否发送成功
    """
    try:
        message_ = [AgentMessageSegment(type=AgentMessageSegmentType.FILE, content=file_path)]
        await chat_service.send_agent_message(chat_key, message_, _ctx, record=True)
    except Exception as e:
        logger.exception(f"Error sending message to chat: {e}")
        return False
    else:
        return True


@agent_collector.mount_method(MethodType.TOOL)
async def get_user_avatar(user_qq: str, _ctx: AgentCtx) -> str:
    """获取用户头像

    Args:
        user_qq (str): 用户 QQ 号

    Returns:
        str: 头像文件路径
    """
    try:
        file_path, file_name = await download_file(f"https://q1.qlogo.cn/g?b=qq&nk={user_qq}&s=640")
        return str(convert_file_name_to_container_path(file_name))
    except Exception as e:
        raise Exception(f"Error getting user avatar: {e}") from e


@agent_collector.mount_method(MethodType.BEHAVIOR)
async def update_preset_status(chat_key: str, setting_name: str, description: str, _ctx: AgentCtx) -> str:
    """更新人物设定状态

    **注意**: 你必须在**当且仅当**场景发展变化导致**不符合**当前设定状态时调用此方法来更新自身状态，但不能过度频繁使用，否则会丢失较早的状态记录!

    Args:
        chat_key (str): 会话标识
        setting_name (str): 新状态下的人设名
        description (str): 变化状态描述与原因 (推荐格式 "由于 [事件]，转变为 [新状态的详细描述] [但仍然... (可选)]" **事件必须基于上下文进行总结描述尽可能详细地说明人物状态、外观、动作等信息，但如果上一状态中的某些信息依然有效，需要追加补充说明以免丢失信息**)

    Returns:
        str: 操作结果

    Example:
        > Since the number of context items is limited, when your dynamic setting state does not match the current scene, you need to call this method to update it.
        ```
        # 假设新状态下的人设名为 "正在认真看书的可洛喵"
        update_preset_status(chat_key, "正在认真看书的可洛喵", "由于 ... 可洛喵 ... 转变为 `正在认真看书的可洛喵`")
        ```
    """
    db_channel: DBChatChannel = DBChatChannel.get_channel(chat_key)
    channel_data: channelData = db_channel.get_channel_data()
    before_status: Optional[PresetStatus] = channel_data.get_latest_preset_status()
    new_status = PresetStatus(setting_name=setting_name, description=description, translated_timestamp=int(time.time()))
    await channel_data.update_preset_status(new_status)
    db_channel.save_channel_data(channel_data)

    return (
        f"Preset status updated from `{before_status.setting_name}` to `{setting_name}`"
        if before_status
        else f"Preset status updated to {setting_name}"
    )



from fastapi import APIRouter, Depends, FastAPI
from fastapi.openapi.docs import get_swagger_ui_html
from fastapi.openapi.utils import get_openapi
from fastapi.responses import JSONResponse
from fastapi.security import OAuth2PasswordRequestForm

from nekro_agent.schemas.message import Ret
from nekro_agent.schemas.user import UserLogin, UserToken
from nekro_agent.services.user import user_login

from .rpc import router as exec_router
from .tools import router as tools_router
from .user import router as user_router


def mount_routers(app: FastAPI):

    api = APIRouter(prefix="/api")

    api.include_router(user_router)
    api.include_router(tools_router)
    api.include_router(exec_router)

    @api.get("/health", response_model=Ret, tags=["Health"], summary="健康检查")
    async def _() -> Ret:
        """测试服务是否正常运行"""
        return Ret.success(msg="Nekro agent Service Running...")

    @api.post("/token", response_model=UserToken, tags=["User"], summary="OpenAPI OAuth2 授权")
    async def _(form_data: OAuth2PasswordRequestForm = Depends()) -> UserToken:
        """登陆获取token"""
        return await user_login(
            UserLogin(username=form_data.username, password=form_data.password),
        )

    # 生成 OpenAPI 文档
    @api.get("/openapi.json", include_in_schema=False)
    async def openapi():
        openapi_schema = get_openapi(
            title="Nekro Agent API",
            version="0.1.0",
            routes=api.routes,
            description="Nekro Agent API 文档",
        )
        return JSONResponse(openapi_schema)

    # 挂载 API 文档
    @api.get("/docs", include_in_schema=False)
    async def custom_swagger_ui_html():
        return get_swagger_ui_html(
            openapi_url="/api/openapi.json",
            title="Nekro Agent API",
            oauth2_redirect_url="/api/token",
        )

    app.include_router(api)



import os
from typing import List, Optional, Tuple, Union

from miose_toolkit_common import Env
from nonebot import on_command
from nonebot.adapters import Bot, Message
from nonebot.adapters.onebot.v11 import GroupMessageEvent, MessageEvent
from nonebot.matcher import Matcher
from nonebot.params import CommandArg

from nekro_agent.core.config import config, reload_config
from nekro_agent.core.logger import logger
from nekro_agent.core.os_env import OsEnv
from nekro_agent.models.db_chat_channel import DBChatChannel
from nekro_agent.models.db_chat_message import DBChatMessage
from nekro_agent.schemas.chat_channel import (
    MAX_PRESET_STATUS_SHOW_SIZE,
    PresetStatus,
    channelData,
)
from nekro_agent.schemas.chat_message import ChatType
from nekro_agent.services.chat import chat_service
from nekro_agent.services.sandbox.executor import limited_run_code
from nekro_agent.systems.message.push_bot_msg import push_system_message
from nekro_agent.tools.common_util import get_app_version
from nekro_agent.tools.onebot_util import gen_chat_text, get_chat_info, get_user_name


async def command_guard(
    event: Union[MessageEvent, GroupMessageEvent],
    bot: Bot,
    arg: Message,
    matcher: Matcher,
) -> Tuple[str, str, str, ChatType]:
    """指令执行前处理

    Args:
        event (Union[MessageEvent, GroupMessageEvent]): 事件对象
        bot (Bot): Bot 对象
        arg (Message): 命令参数
        matcher (Matcher): Matcher 对象

    Returns:
        Tuple[str, ChatType]: 用户名, 命令内容(不含命令名), 会话标识, 会话类型
    """
    username = await get_user_name(event=event, bot=bot, user_id=event.get_user_id())
    # 判断是否是禁止使用的用户
    if event.get_user_id() not in config.SUPER_USERS:
        logger.warning(f"用户 {username} 不在允许用户中")
        await matcher.finish(f"用户 [{event.get_user_id()}]{username} 不在允许用户中")

    cmd_content: str = arg.extract_plain_text().strip()
    chat_key, chat_type = await get_chat_info(event=event)
    return username, cmd_content, chat_key, chat_type


@on_command("reset", priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    target_chat_key: str = cmd_content or chat_key
    if not target_chat_key:
        await matcher.finish(message="请指定要清空聊天记录的会话")
    db_chat_channel: DBChatChannel = DBChatChannel.get_channel(chat_key=target_chat_key)
    db_chat_channel.reset_channel()
    msgs = DBChatMessage.filter(conditions={DBChatMessage.chat_key: target_chat_key})
    msg_cnt = len(msgs)

    for msg in msgs:
        msg.delete()
    await matcher.finish(message=f"已清空 {msg_cnt} 条 {target_chat_key} 的聊天记录")


@on_command("inspect", priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    target_chat_key: str = cmd_content or chat_key
    if not target_chat_key:
        await matcher.finish(message="请指定要查询的会话")
    db_chat_channel: DBChatChannel = DBChatChannel.get_channel(chat_key=target_chat_key)

    info = f"基本人设: {config.AI_CHAT_PRESET_NAME}\n"
    channel_data: channelData = db_chat_channel.get_channel_data()
    if channel_data.preset_status_list:
        info += "人设状态历史:\n"
    for status in channel_data.preset_status_list[-MAX_PRESET_STATUS_SHOW_SIZE:]:
        info += f"[{status.setting_name}] - {status.description}\n"

    await matcher.finish(message=f"频道 {target_chat_key} 信息：\n{info.strip()}")


@on_command("exec", priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    result: str = await limited_run_code(cmd_content, from_chat_key=chat_key)

    if result:
        await matcher.finish(result)


@on_command("system", priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    await push_system_message(chat_key=chat_key, agent_messages=cmd_content)
    await matcher.finish(message="系统消息添加成功")


@on_command("config_show", priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    if not cmd_content:
        modifiable_config_key: List[str] = []
        for _key, _value in config.dump_config_template().items():
            if isinstance(_value, (int, float, bool, str)):
                modifiable_config_key.append(_key)
        sep = "\n"
        await matcher.finish(message=f"当前支持动态修改配置：\n{sep.join([f'- {k}' for k in modifiable_config_key])}")
    else:
        if config.dump_config_template().get(cmd_content):
            await matcher.finish(message=f"当前配置：\n{cmd_content}={getattr(config, cmd_content)}")
        else:
            await matcher.finish(message=f"未知配置 `{cmd_content}`")


@on_command("config_set", priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    try:
        key, value = cmd_content.strip().split("=", 1)
    except ValueError:
        await matcher.finish(message="参数错误，请使用 `config_set key=value` 的格式")

    if config.dump_config_template().get(key):
        _c_type = type(getattr(config, key))
        _c_value = getattr(config, key)
        if isinstance(_c_value, (int, float)):
            setattr(config, key, _c_type(value))
        elif isinstance(_c_value, bool):
            if value.lower() in ["true", "1", "yes"]:
                setattr(config, key, True)
            elif value.lower() in ["false", "0", "no"]:
                setattr(config, key, False)
            else:
                await matcher.finish(message=f"布尔值只能是 `true` 或 `false`，请检查 `{key}` 的值")
        elif isinstance(_c_value, str):
            setattr(config, key, _c_type(value))
        else:
            await matcher.finish(message=f"不支持动态修改的配置类型 `{_c_type}`")
        await matcher.finish(message=f"已设置 `{key}` 的值为 `{value}`")
    else:
        await matcher.finish(message=f"未知配置: `{key}`")


@on_command("config_reload", priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    await matcher.finish(message="功能未实现")
    try:
        config.dump_config(envs=[Env.Default.value])
    except Exception as e:
        await matcher.finish(message=f"保存配置失败：{e}")
    else:
        await matcher.finish(message="已保存配置")


@on_command("config_save", priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    await matcher.finish(message="功能未实现")
    reload_config()
    await matcher.finish(message="重载配置成功")


@on_command("docker_restart", priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    if not OsEnv.RUN_IN_DOCKER:
        await matcher.finish(message="当前环境不在 Docker 容器中，无法执行此操作")

    container_name: str = cmd_content or "nekro_agent"
    os.system(f"docker restart {container_name}")


@on_command("docker_logs", priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    if not OsEnv.RUN_IN_DOCKER:
        await matcher.finish(message="当前环境不在 Docker 容器中，无法执行此操作")

    lines_limit: int = 100
    container_name: str = cmd_content or "nekro_agent"
    logs = os.popen(f"docker logs {container_name} --tail {lines_limit}").read()
    await matcher.finish(message=f"容器日志: \n{logs}")


@on_command("sh", priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    outputs = os.popen(cmd_content).read()
    await matcher.finish(message=f"命令 `{cmd_content}` 输出: \n{outputs or '<Empty>'}")


@on_command("na_info", priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    version: str = get_app_version()
    await matcher.finish(
        message=(
            f"[Nekro-Agent] - 更智能、更优雅的代理执行 AI\n"
            f"Author: KroMiose\n"
            f"Github: https://github.com/KroMiose/nekro-agent\n"
            f"Version: {version}\n"
            f"In-Docker: {OsEnv.RUN_IN_DOCKER}\n"
            "========会话设定========\n"
            f"人设: {config.AI_CHAT_PRESET_NAME}\n"
            f"当前模型组: {config.USE_MODEL_GROUP}\n"
        ).strip(),
    )



import datetime

from miose_toolkit_db import Mapped, MappedColumn, MioModel
from sqlalchemy import Boolean, DateTime, Integer, String, Text, func

from nekro_agent.core.database import orm
from nekro_agent.core.logger import logger
from nekro_agent.schemas.chat_channel import channelData


@orm.reg_predefine_data_model(table_name="chat_channel", primary_key="id")
class DBChatChannel(MioModel):
    """数据库聊天频道模型"""

    id: Mapped[int] = MappedColumn(Integer, primary_key=True, autoincrement=True, comment="ID")

    chat_key: Mapped[str] = MappedColumn(String(32), comment="会话唯一标识", index=True)
    is_active: Mapped[bool] = MappedColumn(Boolean, default=True, comment="是否激活")
    data: Mapped[str] = MappedColumn(Text, comment="频道数据")

    create_time: Mapped[datetime.datetime] = MappedColumn(DateTime, server_default=func.now(), comment="创建时间")
    update_time: Mapped[datetime.datetime] = MappedColumn(
        DateTime,
        server_default=func.now(),
        onupdate=func.now(),
        comment="更新时间",
        index=True,
    )

    @classmethod
    def get_channel(cls, chat_key: str) -> "DBChatChannel":
        """获取聊天频道"""
        assert chat_key, "获取聊天频道失败，chat_key 为空"
        data = cls.filter(conditions={DBChatChannel.chat_key: chat_key}, limit=1)
        return (
            data[0]
            if data
            else cls.add(
                data={DBChatChannel.chat_key: chat_key, DBChatChannel.data: channelData(chat_key=chat_key).model_dump_json()},
            )
        )

    def get_channel_data(self) -> channelData:
        """获取聊天频道数据"""
        try:
            return channelData.model_validate_json(self.data)
        except Exception as e:
            logger.error(f"获取聊天频道数据失败，{e} 重置使用新数据")
            self.reset_channel()
            return channelData(chat_key=self.chat_key)

    def save_channel_data(self, data: channelData):
        """保存聊天频道数据"""
        self.update({DBChatChannel.data: data.model_dump_json()})

    def reset_channel(self):
        """重置聊天频道"""
        self.update({DBChatChannel.data: channelData(chat_key=self.chat_key).model_dump_json()})



import hashlib
import random
import re
from pathlib import Path
from typing import Tuple

import httpx
import toml

from nekro_agent.core.config import config
from nekro_agent.core.os_env import USER_UPLOAD_DIR


def get_app_version() -> str:
    """获取当前应用版本号

    Returns:
        str: 应用版本号
    """
    pyproject = toml.loads(Path("pyproject.toml").read_text())
    try:
        return pyproject["tool"]["poetry"]["version"]
    except KeyError:
        return "unknown"


async def download_file(
    url: str,
    file_path: str = "",
    file_name: str = "",
    use_suffix: str = "",
    retry_count: int = 3,
) -> Tuple[str, str]:
    """下载文件

    Args:
        url (str): 下载链接
        file_path (str): 保存路径

    Returns:
        Tuple[str, str]: 文件路径, 文件名
    """

    try:
        async with httpx.AsyncClient() as client:
            response = await client.get(url)
            response.raise_for_status()
            if not file_path:
                file_name = file_name or f"{hashlib.md5(response.content).hexdigest()}{use_suffix}"
                save_path = Path(USER_UPLOAD_DIR) / Path(file_name)
                save_path.parent.mkdir(parents=True, exist_ok=True)
                file_path = str(save_path)
            Path(file_path).write_bytes(response.content)
            Path(file_path).chmod(0o755)
    except Exception:
        if retry_count > 0:
            return await download_file(url, file_path, file_name, use_suffix, retry_count=retry_count - 1)
        raise
    else:
        return file_path, file_name


async def move_to_upload_dir(file_path: str, file_name: str = "", use_suffix: str = "") -> Tuple[str, str]:
    """复制文件到上传目录

    Args:
        file_path (str): 文件路径
        file_name (str): 文件名

    Returns:
        Tuple[str, str]: 文件路径, 文件名
    """
    if not file_name:
        file_name = f"{hashlib.md5(Path(file_path).read_bytes()).hexdigest()}{use_suffix}"
    save_path = Path(USER_UPLOAD_DIR) / Path(file_name)
    save_path.parent.mkdir(parents=True, exist_ok=True)
    Path(save_path).write_bytes(Path(file_path).read_bytes())
    Path(save_path).chmod(0o755)
    return str(save_path), file_name


def convert_path_to_container_path(path: str) -> Path:
    """将路径转换为容器内路径

    Args:
        path (str): 路径

    Returns:
        Path: 容器内路径
    """

    return Path("/app/uploads") / Path(path).name


def convert_file_name_to_container_path(file_name: str) -> Path:
    """将文件名转换为容器内路径

    Args:
        file_name (str): 文件名

    Returns:
        Path: 容器内路径
    """

    return Path("/app/uploads") / Path(file_name)


def get_downloaded_prompt_file_path(file_name: str) -> Path:
    """获取已下载文件路径

    Args:
        file_name (str): 文件名

    Returns:
        Path: 文件路径
    """

    return "app/uploads" / Path(file_name)


def random_chat_check() -> bool:
    """随机聊天检测

    Returns:
        bool: 是否随机聊天
    """

    return random.random() < config.AI_CHAT_RANDOM_REPLY_PROBABILITY


def check_content_trigger(content: str) -> bool:
    """内容触发检测

    Args:
        content (str): 内容

    Returns:
        bool: 是否触发
    """

    for reg_text in config.AI_CHAT_TRIGGER_REGEX:
        reg = re.compile(reg_text)
        if reg.search(content):
            return True
    return False



from nekro_agent.core.database import orm

from .db_chat_channel import DBChatChannel
from .db_chat_message import DBChatMessage
from .db_user import DBUser


def database_init():
    orm.create_all()



import time
from typing import List, Union

from nekro_agent.core import config, logger
from nekro_agent.models.db_chat_message import DBChatMessage
from nekro_agent.schemas.agent_message import (
    AgentMessageSegment,
    AgentMessageSegmentType,
    convert_agent_message_to_prompt,
)
from nekro_agent.schemas.chat_message import ChatType


async def push_bot_chat_message(chat_key: str, agent_messages: Union[str, List[AgentMessageSegment]]):
    """推送机器人消息"""

    logger.info(f"Pushing Bot Message To Chat {chat_key}")
    if isinstance(agent_messages, str):
        agent_messages = [AgentMessageSegment(type=AgentMessageSegmentType.TEXT, content=agent_messages)]

    send_timestamp = int(time.time())
    content_text = convert_agent_message_to_prompt(agent_messages)

    DBChatMessage.add(
        data={
            DBChatMessage.sender_id: -1,
            DBChatMessage.sender_bind_qq: config.BOT_QQ or "0",
            DBChatMessage.sender_real_nickname: config.AI_CHAT_PRESET_NAME,
            DBChatMessage.sender_nickname: config.AI_CHAT_PRESET_NAME,
            DBChatMessage.is_tome: 0,
            DBChatMessage.is_recalled: False,
            DBChatMessage.chat_key: chat_key,
            DBChatMessage.chat_type: ChatType.from_chat_key(chat_key).value,
            DBChatMessage.content_text: content_text,
            DBChatMessage.content_data: [],
            DBChatMessage.raw_cq_code: "",
            DBChatMessage.ext_data: {},
            DBChatMessage.send_timestamp: send_timestamp,
        },
    )


async def push_system_message(chat_key: str, agent_messages: Union[str, List[AgentMessageSegment]]):
    """推送系统消息"""

    logger.info(f"Pushing System Message To Chat {chat_key}")
    if isinstance(agent_messages, str):
        agent_messages = [AgentMessageSegment(type=AgentMessageSegmentType.TEXT, content=agent_messages)]

    send_timestamp = int(time.time())
    content_text = convert_agent_message_to_prompt(agent_messages)

    DBChatMessage.add(
        data={
            DBChatMessage.sender_id: -1,
            DBChatMessage.sender_bind_qq: "0",
            DBChatMessage.sender_real_nickname: "SYSTEM",
            DBChatMessage.sender_nickname: "SYSTEM",
            DBChatMessage.is_tome: 0,
            DBChatMessage.is_recalled: False,
            DBChatMessage.chat_key: chat_key,
            DBChatMessage.chat_type: ChatType.from_chat_key(chat_key).value,
            DBChatMessage.content_text: content_text,
            DBChatMessage.content_data: [],
            DBChatMessage.raw_cq_code: "",
            DBChatMessage.ext_data: {},
            DBChatMessage.send_timestamp: send_timestamp,
        },
    )



import random

from nonebot.adapters.onebot.v11 import Bot

from nekro_agent.core import logger
from nekro_agent.schemas.agent_ctx import AgentCtx
from nekro_agent.services.chat import chat_service
from nekro_agent.services.extension import ExtMetaData
from nekro_agent.tools.collector import MethodType, agent_collector

__meta__ = ExtMetaData(
    name="dice",
    description="Nekro-Agent 掷骰姬",
    version="0.1.0",
    author="KroMiose",
    url="https://github.com/KroMiose/nekro-agent",
)


@agent_collector.mount_method(MethodType.AGENT)
async def dice_roll(event_name: str, description: str, difficulty: int, _ctx: AgentCtx) -> str:
    """对可能产生不同结果的事件发起掷骰检定请求以确认执行结果 (use lang: zh-CN)

    **应用场景: 战斗、防护、反抗、逃跑、随机事件、行为等 (!你需要在这些场景中积极使用掷骰检定!)**

    Args:
        event_name (str): 事件名称
        description (str): 事件详细描述
        difficulty (int): 事件难度 - 分析场景，给出一个客观合理的预期的难度值 (范围: 1-20)
    """

    assert 1 <= difficulty <= 20, "Difficulty should be between 1 and 20"

    def get_result_str(roll_result: int, difficulty: int) -> str:
        if roll_result == 1 and roll_result < difficulty:
            return "大失败!"
        if roll_result == 20 and roll_result >= difficulty:
            return "大成功!"
        if roll_result < difficulty:
            return "失败"
        if roll_result >= difficulty:
            return "成功"
        raise ValueError("Invalid roll result")

    roll_result = random.randint(1, 20)
    result_str = get_result_str(roll_result, difficulty)
    await chat_service.send_message(
        _ctx.from_chat_key,
        f"【检定事件】{event_name} ({difficulty}/20)\n> {description}\n========\n掷骰结果：{result_str} 【{roll_result}】",
    )
    return (
        f"[{event_name}] ({difficulty}/20) roll result: {roll_result}【{result_str}】\n"
        "Please continue to generate responses based on the results"
    )



import os
import sys
from pathlib import Path
from typing import Any, Tuple


class ArgTypes:

    @staticmethod
    def _use_arg(arg_key: str, default: Any = None) -> Any:
        if arg_key in sys.argv:
            index = sys.argv.index(arg_key)
            if index + 1 < len(sys.argv):
                return sys.argv[index + 1]
        return default

    @staticmethod
    def Str(key: str, default: str = "") -> str:
        return str(ArgTypes._use_arg(key, default))

    @staticmethod
    def Int(key: str, default: int = 0) -> int:
        return int(ArgTypes._use_arg(key, default))

    @staticmethod
    def Float(key: str, default: float = 0.0) -> float:
        return float(ArgTypes._use_arg(key, default))

    @staticmethod
    def Bool(key: str) -> bool:
        return key in sys.argv


class OsEnvTypes:

    @staticmethod
    def _use_env(env_key: str, default: Any = None) -> Any:
        return os.environ.get(f"NEKRO_{env_key.upper()}", default)

    @staticmethod
    def Str(key: str, default: str = "") -> str:
        return str(OsEnvTypes._use_env(key, default))

    @staticmethod
    def Int(key: str, default: int = 0) -> int:
        return int(OsEnvTypes._use_env(key, default))

    @staticmethod
    def Float(key: str, default: float = 0.0) -> float:
        return float(OsEnvTypes._use_env(key, default))

    @staticmethod
    def Bool(key: str) -> bool:
        return str(OsEnvTypes._use_env(key, "false")).lower() == "true"



import asyncio
import inspect

import nekro_agent.services.sandbox.ext_caller_code
from nekro_agent.core import logger
from nekro_agent.tools.collector import agent_collector

CODE_PREAMBLE = """
from api_caller import *
"""

METHOD_REG_TEMPLATE = """
@__extension_method_proxy
def {method_name}(*args, **kwargs):
    pass
"""  #! 沙盒环境下不需要使用异步方式调用，因为实际执行是通过 RPC 调用的


def get_api_caller_code(container_key: str, from_chat_key: str):
    base_code = (
        inspect.getsource(nekro_agent.services.sandbox.ext_caller_code)
        .replace(
            "{CHAT_API}",
            "http://host.docker.internal:8021/api",
        )
        .replace(
            "{CONTAINER_KEY}",
            container_key,
        )
        .replace(
            "{FROM_CHAT_KEY}",
            from_chat_key,
        )
    )
    methods = agent_collector.get_all_methods()

    for method in methods:
        base_code += METHOD_REG_TEMPLATE.format(method_name=method.__name__)
    return base_code.strip()



import asyncio
import re
import time
from typing import Dict

from miose_toolkit_llm.exceptions import ResolveError

from nekro_agent.core import config, logger
from nekro_agent.models.db_chat_message import DBChatMessage
from nekro_agent.schemas.chat_message import ChatMessage
from nekro_agent.services.agents.chat_agent import agent_run
from nekro_agent.tools.common_util import check_content_trigger, random_chat_check

running_chat_task_map: Dict[str, asyncio.Task] = {}
running_chat_task_throttle_map: Dict[str, float] = {}


def message_validation_check(message: ChatMessage) -> bool:
    """消息校验"""

    plaint_text = message.content_text.strip().replace(" ", "").lower()

    # 检查伪造消息
    if re.match(r"<.{4,12}\|messageseparator>", plaint_text):
        return False
    if re.match(r"<.{4,12}\|messageseperator>", plaint_text):
        return False

    if "message" in plaint_text and "(qq:" in plaint_text:
        return False
    if "from_qq:" in plaint_text:
        return False

    return True


async def push_human_chat_message(message: ChatMessage):
    """推送聊天消息"""
    global running_chat_task_map, running_chat_task_throttle_map

    logger.info(f'Message Received: "{message.content_text}" From {message.sender_real_nickname}')

    if not message_validation_check(message):
        logger.warning("消息校验失败，跳过本次处理...")
        return

    content_data = [o.model_dump() for o in message.content_data]

    DBChatMessage.add(
        data={
            DBChatMessage.sender_id: message.sender_id,
            DBChatMessage.sender_bind_qq: message.sender_bind_qq,
            DBChatMessage.sender_real_nickname: message.sender_real_nickname,
            DBChatMessage.sender_nickname: message.sender_nickname,
            DBChatMessage.is_tome: message.is_tome,
            DBChatMessage.is_recalled: message.is_recalled,
            DBChatMessage.chat_key: message.chat_key,
            DBChatMessage.chat_type: message.chat_type,
            DBChatMessage.content_text: message.content_text,
            DBChatMessage.content_data: content_data,
            DBChatMessage.raw_cq_code: message.raw_cq_code,
            DBChatMessage.ext_data: message.ext_data,
            DBChatMessage.send_timestamp: message.send_timestamp,
        },
    )

    if (
        config.AI_CHAT_PRESET_NAME in message.content_text  # 提及人设名
        or message.is_tome  # 引用 / @ 回复
        or random_chat_check()  # 随机聊天
        or check_content_trigger(message.content_text)  # 触发词
    ):
        if message.chat_key in running_chat_task_throttle_map:
            current_time = time.time()
            running_chat_task_throttle_map[message.chat_key] = current_time
            await asyncio.sleep(config.AI_GENERATE_THROTTLE_SECONDS)
            if running_chat_task_throttle_map[message.chat_key] != current_time:
                logger.warning("检测到高频触发消息，节流控制生效中，跳过本次处理...")
                return
        if message.chat_key in running_chat_task_map:
            logger.info(f"检测到正在进行的聊天任务: {message.chat_key} 取消之前的任务")
            running_chat_task_map[message.chat_key].cancel()
        running_chat_task_map[message.chat_key] = asyncio.create_task(agent_task(message))


async def agent_task(message: ChatMessage):
    global running_chat_task_map
    logger.info(f"Message From {message.sender_real_nickname} is ToMe, Running Chat Agent...")
    for i in range(3):
        try:
            await agent_run(message)
        except ResolveError:
            logger.error(f"Resolve Error, Retrying {i+1}/3...")
        else:
            break
    else:
        logger.error("Failed to Run Chat Agent.")

    del running_chat_task_map[message.chat_key]
